%let version=1.0;
*********************************************************************************************************************************************;
*** ACS Linkage Program - Version 1.0                                                                                                        ;
/** SAS program to ACS variables based upon county and census tract information in registry incidence data.
    The program will generate 2 blocks of ACS data (2007-2011, 2011-2015).
    
    THIS PROGRAM DOES NOT BLANK OUT CENSUS TRACT 2010!!!!  
    YOU SHOULD RUN THIS PROGRAM BEFORE RUNNING THE CENSUS TRACT POVERTY LINKAGE SAS PROGRAM - WHICH DOES BLANK OUT THE CENSUS TRACT(S).

    Name of file containing poverty group information by state, county, and census tract:
         acs_2007_2011.dat
         acs_2011_2015.dat

    NOTE: The following list identifies counties with possible geographic code errors or definition differences for tracts and block groups. 
    Counties in this list may generate an "Unknown" value as a result.
         Alabama: Bibb
         Illinois: Coles
         Iowa: Adair, Adams, Cass, Clay, Dickinson, Dubuque, Emmet, Page, Sioux, Union, Worth
         Maryland: Calvert
         Massachusetts: Berkshire
         Michigan: Baraga
         Minnesota: Murray
         Texas: Wilson

         *For more information please see http://www.census.gov/acs/www/data_documentation/geography_notes/#tracts

    Written for NAACCR, Inc.
    October 2017
**/
*********************************************************************************************************************************************;
**** Written/modified:                                                                                                                       ;
****    William Howe, Information Management Services, Inc., 301.680.9770                                                                    ;
*********************************************************************************************************************************************;

*********************************************************************************************************************************************;
/*** NAACCR Version 16.0 input registry file, with the following fields included:
       Patient ID [item #20]
       State at DX [item #80]
       County at DX [item #90]
       Census Tract 2010 [item #135]
       Date of Diagnosis [item #390]
***/;

***filename regfile 'directory and file name';
filename regfile './sample.txt';

*** Output file Name - Text file, with Patient ID, state, county, census tract, census block, and generated Percent Poverty values;
*** filename outfile 'directory and file name' ;
filename outfile './out.txt';
*********************************************************************************************************************************************;

data _null_;
  currentYear = year(date());
  call symput('currentYear', currentYear);
run;

************************************************************************************************************************************************;
***** Read in the ACS data files ***************************************************************************************************************;
************************************************************************************************************************************************;
data acs_0711;
  infile "acs_2007_2011.dat" lrecl=364;
  input  @001 acs_key  $char11.
         @012 acs_0711 $char352.;
run;
data acs_1115;
  infile "acs_2011_2015.dat" lrecl=454;
  input  @001 acs_key  $char11.
         @012 acs_1115 $char442.;
run;
proc sort data = acs_0711; by acs_key; run;
proc sort data = acs_1115; by acs_key; run;

************************************************************************************************************************************************;
***** Read in data file, convert two letter state code to a 2 digit FIPS code, apply format to state/county/census tract, and write out records ;
************************************************************************************************************************************************;
data registry_data;
  infile regfile lrecl=3339 pad;
  
  input  @001 Incidence_record $char3339.
         @042 DataItem_20      $char8.
         @145 DataItem_80      $char2.
         @156 DataItem_90      $char3.
         @428 DataItem_135     $char6.
         @530 DataItem_390     $char8.;
         
  year_of_diag = input(substr(DataItem_390,1,4),4.);
  acs_key  = DataItem_80||DataItem_90||DataItem_135;
run;
proc sort data = registry_data; by acs_key; run;

************************************************************************************************************************************************;
***** Merge the data ***************************************************************************************************************************;
************************************************************************************************************************************************;
data registry_data;
	merge registry_data (in=a) acs_0711 (in=b);
	by acs_key;
	if a;
run;

data registry_data;
	merge registry_data (in=a) acs_1115 (in=b);
	by acs_key;
	if a;
run;

data registry_data;
	set registry_data;
  if not (2005 <= year_of_diag <= &currentYear.) then do;
     acs_0711 = repeat(' ',351);
     acs_1115 = repeat(' ',441);
  end;
  
  if trim(acs_0711) = '' then flag_0711 = 0; else flag_0711 = 1;
  if trim(acs_1115) = '' then flag_1115 = 0; else flag_1115 = 1;

  label Incidence_record = 'Incidence Record'
        DataItem_20      = 'Patient ID Number'
        DataItem_80      = 'Addr at DX - State'
        DataItem_90      = 'County at DX'
        DataItem_135     = 'Census Tract 2010'
        DataItem_390     = 'Date of Diagnosis'
        acs_key          = 'State/County/Tract Lookup'
        acs_0711         = 'ACS Data (2007-2011)'
        acs_1115         = 'ACS Data (2011-2015)'
        flag_0711        = 'ACS Data (2007-2011) Linkage Flag'
        flag_1115        = 'ACS Data (2011-2015) Linkage Flag'
        year_of_diag     = 'Year of Diagnosis'; 
run;

/* Create a new file with ACS data */
data _null_;
  set registry_data;
  file outfile lrecl=3339 pad;
  put @001  Incidence_record $char3339.
      @2512 acs_0711 $char352.
      @2864 acs_1115 $char442.;
  run;

****** Summarize Results;
proc format;
 value flag
       0= 'Not Linked'
       1= 'Linked';
 run;

proc freq data=registry_data;
  tables flag_0711 * year_of_diag / list missing;
  title "NAACCR - ACS Assignment Program version &version.";
  title2 'Frequency Distribution for ACS 2007-2011 Flag';
  format flag_0711 flag.;
run;

proc freq data=registry_data;
  tables flag_1115 * year_of_diag / list missing;
  title "NAACCR - ACS Assignment Program version &version.";
  title2 'Frequency Distribution for ACS 2011-2015 Flag';
  format flag_1115 flag.;
run;